#include "cfg-cmd.h"

#include "scm-mem.h"
#include <string.h>
#include <stdio.h>

// -------------------------------------------------------------------------------------------- //
// * Implementation *
// -------------------------------------------------------------------------------------------- //

scm::cfgCommand::cfgCommand()
{
	nalloc = 0;
	narg = 0;

	name_memsize = 1;
	name = new char[name_memsize];
	*name = '\0';
}

scm::cfgCommand::~cfgCommand()
{
	if (nalloc > 0) delete[] arg;
	delete[] name;

	narg = 0;
	nalloc = 0;
}

scm::cfgCommand::cfgCommand(const cfgCommand& cmd)
{
	name_memsize = strlen(cmd.name) + 1;
	name = new char[name_memsize];
	strcpy(name, cmd.name);

	narg = cmd.narg;
	if (narg > 0) {
		arg = new cfgVector[narg];
		for (int k = 0; k < narg; k++)
			arg[k] = cmd.arg[k];
	}

	nalloc = narg;
}
// -------------------------------------------------------------------------------------------- //

// * swap & assignment * //
// -------------------------------------------------------------------------------------------- //
const scm::cfgCommand&
scm::cfgCommand::operator=(cfgCommand cmd)
{
	swap(cmd);
	return (*this);
}

void scm::cfgCommand::swap(
	cfgCommand& cmd)
{
	scm::swap(narg, cmd.narg);
	scm::swap(nalloc, cmd.nalloc);
	scm::swap(arg, cmd.arg);

	scm::swap(name, cmd.name);
	scm::swap(name_memsize, cmd.name_memsize);
}
// -------------------------------------------------------------------------------------------- //

// * add argument * //
// -------------------------------------------------------------------------------------------- //
bool scm::cfgCommand::add_arg(const cfgVector& _arg)
{
	if (!_arg.is_defined()) return false;

	if (narg == nalloc) {

		cfgVector *arg_list;
		arg_list = new cfgVector[nalloc + c_nalloc_inc];

		for (int k = 0; k < narg; k++)
			arg_list[k] = arg[k];

		if (nalloc > 0) delete[] arg;
		arg = arg_list;

		nalloc += c_nalloc_inc;
	}

	arg[narg] = _arg;
	narg++;

	return true;
}
// -------------------------------------------------------------------------------------------- //

// * change command name * //
// -------------------------------------------------------------------------------------------- //
void scm::cfgCommand::set_name(const char* ex_name)
{
	if (ex_name == NULL) return;

	const size_t ex_length = strlen(ex_name) + 1;
	if (ex_length > name_memsize) {
		delete[] name;

		name_memsize = ex_length;
		name = new char[name_memsize];
	}

	strcpy(name, ex_name);
}
// -------------------------------------------------------------------------------------------- //

// * reset command * //
// -------------------------------------------------------------------------------------------- //
void scm::cfgCommand::reset()
{
	narg = 0;
	*name = '\0';
}
// -------------------------------------------------------------------------------------------- //

// * get & check calls * //
// -------------------------------------------------------------------------------------------- //
int scm::cfgCommand::get_narg() const
{
	return narg;
}

scm::cfgVector scm::cfgCommand::get_arg(const int idx) const
{
	if ((idx < 0) || (idx >= narg)) return cfgVector();
	return arg[idx];
}

bool scm::cfgCommand::is_name_eq(const char* ex_name) const
{
	if (ex_name == NULL) return false;
	return (!strcmp(ex_name, name));
}

const char* scm::cfgCommand::get_name() const
{
	return name;
}
// -------------------------------------------------------------------------------------------- //

// * print * //
// -------------------------------------------------------------------------------------------- //
void scm::cfgCommand::print() const
{
	printf(" > COMMAND '%s', narg = %i\n", name, narg);
	for (int k = 0; k < narg; k++) {
		printf(" %i", k);
		arg[k].print();
	}
}
// -------------------------------------------------------------------------------------------- //
